function [a,gradF,n_n,ad,abc,r_p]=dvl_approx_beta(a_dvl,eta,nu,Theta_dvl)
% Altitude Estimation from DVL
% 
% Purpose: 
%           Use DVL measurements to estimate altitude
%
%    
%               
%                  
% Record of revisions:
% Date:        Programmer:                    Description of change:
%--------------------------------------------------------------------------
% 12 Sep 12    Fredrik Dukan                  org.
% 09 Jan 13    Fredrik Dukan                  incl. DVL orientation 
% 10 jan 13    Fredrik Dukan                  organize result data into matrices
% 15 jan 13    Fredrik Dukan                  Least squares plane approx.
% 16 Jan 13    Fredrik Dukan                  added mapping for real data
%
%--------------------------------------------------------------------------
%
% Note: this is for post processing of altitude estimated beased on DVL
% measurements. Based on dvl_alt_beta

%% DVL altitude estimation:
l_data=length(a_dvl);
% map DVL no to quad no: 1->2, 2->4, 3->3, 4->1: (red 300kHz DVL) for
% logfiles from Oct 2011 and earlier. Changes DVL in Nov 2011.
% map DVL no to quad no: 1->2, 2->4, 3->1, 4->3: (blue 600kHz DVL)
%a_dvl=[dvl(:,4) dvl(:,1) dvl(:,3) dvl(:,2)]'; %(red 300kHz DVL) %mapping done

%a_dvl=[dvl(:,2) dvl(:,4) dvl(:,1) dvl(:,3)]'; %mapping done /15.02.12 (blue 600kHz DVL))

%a_dvl=dvl'; %no mapping needed

% Mapping from RDI beam numbering to quadrant number
a_dvl=[a_dvl(3,:);
       a_dvl(2,:);
       a_dvl(4,:);
       a_dvl(1,:)]; %mapping done 22.11.12 (blue 600kHz DVL))

%DVL props:
% gamma=pi/6;
% beta=zeros(4,1);
% tancos=zeros(3,4);

% for i=1:4
% beta(i)=pi/2*(i-1/2);
% tancos(:,i)=[tan(gamma)*cos(beta(i));tan(gamma)*sin(beta(i));1];
% end
% beam alpha and beta angles on DVL:
tancos=[sqrt(3)/3     0      -sqrt(3)/3      0;
        0         sqrt(3)/3     0        -sqrt(3)/3;
        1             1         1            1]; % hard coded! FD oct 12

%Theta_dvl=alignment*pi/180;

% Transformation matrix (DVL-ROV):
Rx_phi2=[1 0 0;
     0 cos(Theta_dvl(1)) -sin(Theta_dvl(1));
     0 sin(Theta_dvl(1)) cos(Theta_dvl(1))];

Ry_theta2=[cos(Theta_dvl(2)) 0 sin(Theta_dvl(2));
        0 1 0;
        -sin(Theta_dvl(2)) 0 cos(Theta_dvl(2))];
 
Rz_psi2=[cos(Theta_dvl(3)) -sin(Theta_dvl(3)) 0;
     sin(Theta_dvl(3)) cos(Theta_dvl(3)) 0;
     0 0 1]; 
 
Rnb_dvl=Rz_psi2*Ry_theta2*Rx_phi2;  % Rotation matrix ROV body frame to NED (NOTE: need only calc this once)
%--------------------------------------------------------------------------

r_dvl=zeros(3,l_data,4);

% translate to ROV frame:
r_off=[0;0;0]; %note this 

r_rov=zeros(3,l_data,4);

r_n=zeros(3,l_data,4);
r_p=zeros(3,l_data,4);

for i=1:4
for j=1:l_data
r_dvl(:,j,i)=a_dvl(i,j)*tancos(:,i);  

r_rov(:,j,i)=Rnb_dvl*r_dvl(:,j,i)+r_off;

% Check if conversion to rads is needed?
phi_hat=0;
theta_hat=0;
psi_hat=eta(4,j);

Rx_phi=[1 0 0;
     0 cos(phi_hat) -sin(phi_hat);
     0 sin(phi_hat) cos(phi_hat)];

Ry_theta=[cos(theta_hat) 0 sin(theta_hat);
        0 1 0;
        -sin(theta_hat) 0 cos(theta_hat)];
 
Rz_psi=[cos(psi_hat) -sin(psi_hat) 0;
     sin(psi_hat) cos(psi_hat) 0;
     0 0 1]; 
 
Rnb=Rz_psi*Ry_theta*Rx_phi; 
 
r_n(:,j,i)=Rnb*r_rov(:,j,i);
r_p(:,j,i)=r_n(:,j,i)+eta(1:3,j);  % coord. of beam intersection points (15.jan 13) changed 16 jan.

end %j

end %i
% Calculate seabed normal vector in body (ROV) frame:

n_n=zeros(3,l_data,4);

for i=1:4
for j=1:l_data %normal vectors of sea floor
   
    if i==1
    n_n(:,j,i)=cross((r_n(:,j,i)-r_n(:,j,i+1)),(r_n(:,j,4)-r_n(:,j,i+1))); 
    elseif i==4
    n_n(:,j,i)=cross((r_n(:,j,i)-r_n(:,j,1)),(r_n(:,j,i-1)-r_n(:,j,1)));  
    else
    n_n(:,j,i)=cross((r_n(:,j,i)-r_n(:,j,i+1)),(r_n(:,j,i-1)-r_n(:,j,i+1)));    
    end
end %j
end %i

%Gradient components of F(x,y,z)=0. seabed
df_x=zeros(4,l_data);
df_y=zeros(4,l_data);

a=zeros(4,l_data); %scalar altitude

tol=5*10^-12;
for i=1:4
for j=1:l_data

check=abs(n_n(3,j,i));

df_x(i,j)=-n_n(1,j,i)/n_n(3,j,i);
df_y(i,j)=-n_n(2,j,i)/n_n(3,j,i);

if check<tol
df_x(i,j)=0;
df_y(i,j)=0;
end

a(i,j)=r_n(3,j,i)-df_x(i,j)*r_n(1,j,i)-df_y(i,j)*r_n(2,j,i);

end
end %i

% %% Least squares estimate--------------------------------------------------
% % get intersection points in NED:
% 
% data=r_p(:,1,1:4);
% 
% 
% n=4;
% A=zeros(n,3); % quadratic approx (3 linear)
% b=zeros(n,1);
%  
% % Set up linear system:
% for j=1:n
%    A(j,1)=1;
%    A(j,2)=data(1,1,j);
%    A(j,3)=data(2,1,j);
% %    A(j,4)=data(1,1,j).^2;
% %    A(j,5)=data(2,1,j).^2;
%    b(j)=data(3,1,j);
% end
% 
% ATA=A'*A;
% ATb=A'*b;
% 
% %solve system
% abc=ATA\ATb;
abc=0;

% end LS estimates--------------------------------------------------------
% altitude derivatives:

ad=zeros(4,l_data); %scalar altitude rate

gradF=zeros(l_data,3,4);

for i=1:4
for j=1:l_data
phi_hat=0;
theta_hat=0;
psi_hat=eta(4,j);

Rx_phi=[1 0 0;
     0 cos(phi_hat) -sin(phi_hat);
     0 sin(phi_hat) cos(phi_hat)];

Ry_theta=[cos(theta_hat) 0 sin(theta_hat);
        0 1 0;
        -sin(theta_hat) 0 cos(theta_hat)];
 
Rz_psi=[cos(psi_hat) -sin(psi_hat) 0;
     sin(psi_hat) cos(psi_hat) 0;
     0 0 1]; 
 
Rnb=Rz_psi*Ry_theta*Rx_phi; 
  
gradF(j,:,i)=[df_x(i,j) df_y(i,j) -1];

%v=[EstStates(j,5);EstStates(j,6);EstStates(j,7)]; %v=[u v w]'
v=nu(1:3,j);
ad(i,j)=gradF(j,:,i)*Rnb*v;

end %j
end %i