function [altitudes,nok]=dvl_kf_beta(a_dvl,eta,nu,offset,alignment)
% Altitude Estimation from DVL
% 
% Purpose: 
%           Use DVL measurements to estimate altitude
%
%    
%               
%                  
% Record of revisions:
% Date:        Programmer:                    Description of change:
%--------------------------------------------------------------------------
% 01 Feb 12    Fredrik Dukan                  org.
% 03 May 12    Fredrik Dukan                  "cosmetic changes"
% Dec 12       Fredrik Dukan                  "avg. gradient"
%
%--------------------------------------------------------------------------
%
% Note: this is for post processing of altitude estimated beased on DVL
% measurements

%% DVL altitude estimation:
l_data=length(a_dvl);
% map DVL no to quad no: 1->2, 2->4, 3->3, 4->1: (red 300kHz DVL) for
% logfiles from Oct 2011 and earlier. Changes DVL in Nov 2011.
% map DVL no to quad no: 1->2, 2->4, 3->1, 4->3: (blue 600kHz DVL)
%a_dvl=[dvl(:,4) dvl(:,1) dvl(:,3) dvl(:,2)]'; %(red 300kHz DVL) %mapping done

% Mapping from RDI beam numbering to quadrant number
a_dvl=[a_dvl(3,:);
       a_dvl(2,:);
       a_dvl(4,:);
       a_dvl(1,:)]; %mapping done 22.11.12 (blue 600kHz DVL))

%a_dvl=raw.DVL.altitudes;

% %DVL props:
% gamma=pi/6;
% beta=zeros(4,1);
% tancos=zeros(3,4);
% 
% for i=1:4 %valid for RDI Workhorse navigator
% beta(i)=pi/2*(i-1/2);
% tancos(:,i)=[tan(gamma)*cos(beta(i));tan(gamma)*sin(beta(i));1];
% end

% beam alpha and beta angles on DVL:
tancos=[sqrt(3)/3     0      -sqrt(3)/3      0;
        0         sqrt(3)/3     0        -sqrt(3)/3;
        1             1         1            1]; % hard coded! FD oct 12

Theta_dvl=alignment*pi/180;

% Transformation matrix (DVL-ROV):
Rx_phi2=[1 0 0;
     0 cos(Theta_dvl(1)) -sin(Theta_dvl(1));
     0 sin(Theta_dvl(1)) cos(Theta_dvl(1))];

Ry_theta2=[cos(Theta_dvl(2)) 0 sin(Theta_dvl(2));
        0 1 0;
        -sin(Theta_dvl(2)) 0 cos(Theta_dvl(2))];
 
Rz_psi2=[cos(Theta_dvl(3)) -sin(Theta_dvl(3)) 0;
     sin(Theta_dvl(3)) cos(Theta_dvl(3)) 0;
     0 0 1]; 
 
Rnb_dvl=Rz_psi2*Ry_theta2*Rx_phi2;  % Rotation matrix ROV body frame to NED (NOTE: need only calc this once)

r_dvl_1=zeros(3,l_data);
r_dvl_2=zeros(3,l_data);
r_dvl_3=zeros(3,l_data);
r_dvl_4=zeros(3,l_data);

% translate to ROV frame:
r_off=offset;

r_rov_1=zeros(3,l_data);
r_rov_2=zeros(3,l_data);
r_rov_3=zeros(3,l_data);
r_rov_4=zeros(3,l_data);

r_n=zeros(3,l_data);
r_n_1=zeros(3,l_data);
r_n_2=zeros(3,l_data);
r_n_3=zeros(3,l_data);
r_n_4=zeros(3,l_data);

DVL_OK=zeros(4,l_data);
nok=zeros(1,l_data);

for j=1:l_data
    
% check if DVL altitude measurements are valid:  --------------------------
for i=1:4    
    if a_dvl(i,j)<0.01  % ad hoc error    
    DVL_OK(i,j)=0;
    %a_dvl(i,j)=a_dvl(i,j-1); % possible fix
    else
    DVL_OK(i,j)=1;   % 1 is OK measurements 
    nok(j)=nok(j)+1; % number of accepted beams
    end
    
end
% end valididty check------------------------------------------------------
        
r_dvl_1(:,j)=a_dvl(1,j)*tancos(:,1);
r_dvl_2(:,j)=a_dvl(2,j)*tancos(:,2);
r_dvl_3(:,j)=a_dvl(3,j)*tancos(:,3);
r_dvl_4(:,j)=a_dvl(4,j)*tancos(:,4);

r_rov_1(:,j)=Rnb_dvl*r_dvl_1(:,j)+r_off;
r_rov_2(:,j)=Rnb_dvl*r_dvl_2(:,j)+r_off;
r_rov_3(:,j)=Rnb_dvl*r_dvl_3(:,j)+r_off;
r_rov_4(:,j)=Rnb_dvl*r_dvl_4(:,j)+r_off;

phi_hat=eta(4,j);
theta_hat=eta(5,j);
psi_hat=eta(6,j);

Rx_phi=[1 0 0;
     0 cos(phi_hat) -sin(phi_hat);
     0 sin(phi_hat) cos(phi_hat)];

Ry_theta=[cos(theta_hat) 0 sin(theta_hat);
        0 1 0;
        -sin(theta_hat) 0 cos(theta_hat)];
 
Rz_psi=[cos(psi_hat) -sin(psi_hat) 0;
     sin(psi_hat) cos(psi_hat) 0;
     0 0 1]; 
 
Rnb=Rz_psi*Ry_theta*Rx_phi; 
  
r_n_1(:,j)=Rnb*r_rov_1(:,j);
r_n_2(:,j)=Rnb*r_rov_2(:,j);
r_n_3(:,j)=Rnb*r_rov_3(:,j); 
r_n_4(:,j)=Rnb*r_rov_4(:,j); 
 
end

% Calculate seabed normal vector in body (ROV) frame:
n_n=zeros(3,l_data);
n_n_1=zeros(3,l_data);
n_n_2=zeros(3,l_data);
n_n_3=zeros(3,l_data);
n_n_4=zeros(3,l_data);

% start error check--------------------------------------------------------
a_select=1; % use beams in  quadrant no. 1,2,4
no_good=0;

for j=1:l_data %normal vectors of sea floor

if nok(j)==4 % 4 available beams
    if  a_select==1 %
        n_n(:,j)=cross((r_n_1(:,j)-r_n_2(:,j)),(r_n_4(:,j)-r_n_2(:,j))); %1
        r_n(:,j)=r_n_1(:,j);
    elseif  a_select==4%
        n_n(:,j)=cross((r_n_4(:,j)-r_n_1(:,j)),(r_n_3(:,j)-r_n_1(:,j))); %4
        r_n(:,j)=r_n_4(:,j);
    elseif  a_select==2 %
        n_n(:,j)=cross((r_n_2(:,j)-r_n_3(:,j)),(r_n_1(:,j)-r_n_3(:,j))); %2
        r_n(:,j)=r_n_2(:,j);
    elseif a_select==3
        n_n(:,j)=cross((r_n_3(:,j)-r_n_4(:,j)),(r_n_2(:,j)-r_n_4(:,j))); %3 
        r_n(:,j)=r_n_3(:,j);
    elseif a_select==0 % mean
        n_n_1(:,j)=cross((r_n_1(:,j)-r_n_2(:,j)),(r_n_4(:,j)-r_n_2(:,j)));
        n_n_2(:,j)=cross((r_n_2(:,j)-r_n_3(:,j)),(r_n_1(:,j)-r_n_3(:,j)));
        n_n_3(:,j)=cross((r_n_3(:,j)-r_n_4(:,j)),(r_n_2(:,j)-r_n_4(:,j)));
        n_n_4(:,j)=cross((r_n_4(:,j)-r_n_1(:,j)),(r_n_3(:,j)-r_n_1(:,j)));
    else
        n_n(:,j)=cross((r_n_1(:,j)-r_n_2(:,j)),(r_n_4(:,j)-r_n_2(:,j))); %1
        r_n(:,j)=r_n_1(:,j);
    end
    no_good=0;

elseif nok(j)==3 % 3 available beams
    if DVL_OK(1,j)==0
        n_n(:,j)=cross((r_n_3(:,j)-r_n_4(:,j)),(r_n_2(:,j)-r_n_4(:,j))); %3 
        r_n(:,j)=r_n_3(:,j);
    elseif DVL_OK(2,j)==0
        n_n(:,j)=cross((r_n_4(:,j)-r_n_1(:,j)),(r_n_3(:,j)-r_n_1(:,j))); %4
        r_n(:,j)=r_n_4(:,j);
    elseif DVL_OK(3,j)==0
        n_n(:,j)=cross((r_n_1(:,j)-r_n_2(:,j)),(r_n_4(:,j)-r_n_2(:,j))); %1
        r_n(:,j)=r_n_1(:,j);
    else
        n_n(:,j)=cross((r_n_2(:,j)-r_n_3(:,j)),(r_n_1(:,j)-r_n_3(:,j))); %2
        r_n(:,j)=r_n_2(:,j);
    end
    no_good=0;

else % less than 3 (1 effectively) available beams (This case should be improved!)
    no_good=no_good+1;
    if no_good>20  %changed tp greater than 20, 5 oct 12. 
        n_n(:,j)=[0;0;-1];
        r_n(:,j)=r_n(:,j-1);
    else
        if j>1
        n_n(:,j)=n_n(:,j-1);  
        r_n(:,j)=r_n(:,j-1);
        else
        n_n(:,j)=[0;0;-1];%n_n(:,j-1);  %mod 22.nov 2012 tp 0 (FD)
        r_n(:,j)=[0;0;0];%r_n(:,j-1);
        end
    end
    
end % 4 available beams

end % normal vectors of sea floor

%-----------end error check------------------------------------------------

for j=1:l_data %normal vectors of sea floor
n_n_1(:,j)=cross((r_n_1(:,j)-r_n_2(:,j)),(r_n_4(:,j)-r_n_2(:,j)));
n_n_2(:,j)=cross((r_n_2(:,j)-r_n_3(:,j)),(r_n_1(:,j)-r_n_3(:,j)));
n_n_3(:,j)=cross((r_n_3(:,j)-r_n_4(:,j)),(r_n_2(:,j)-r_n_4(:,j)));
n_n_4(:,j)=cross((r_n_4(:,j)-r_n_1(:,j)),(r_n_3(:,j)-r_n_1(:,j)));
end

%Gradient components of F(x,y,z)=0. seabed
df_x=zeros(1,l_data);
df_x_1=zeros(1,l_data);
df_x_2=zeros(1,l_data);
df_x_3=zeros(1,l_data);
df_x_4=zeros(1,l_data);

df_y=zeros(1,l_data);
df_y_1=zeros(1,l_data);
df_y_2=zeros(1,l_data);
df_y_3=zeros(1,l_data);
df_y_4=zeros(1,l_data);

a=zeros(1,l_data); %scalar altitude
a_1=zeros(1,l_data); %scalar altitude
a_2=zeros(1,l_data);
a_3=zeros(1,l_data);
a_4=zeros(1,l_data);

tol=5*10^-12;
for j=1:l_data

check_1=abs(n_n_1(3,j));
check_2=abs(n_n_2(3,j));
check_3=abs(n_n_3(3,j));
check_4=abs(n_n_4(3,j));

df_x_1(j)=-n_n_1(1,j)/n_n_1(3,j); %NOTE: this was corrected 15.02.12 from +
df_x_2(j)=-n_n_2(1,j)/n_n_2(3,j);
df_x_3(j)=-n_n_3(1,j)/n_n_3(3,j);
df_x_4(j)=-n_n_4(1,j)/n_n_4(3,j);

df_y_1(j)=-n_n_1(2,j)/n_n_1(3,j);
df_y_2(j)=-n_n_2(2,j)/n_n_2(3,j);
df_y_3(j)=-n_n_3(2,j)/n_n_3(3,j);
df_y_4(j)=-n_n_4(2,j)/n_n_4(3,j); %end corr.

if check_1<tol
df_x_1(j)=0;
df_y_1(j)=0;
end

if check_2<tol
df_x_2(j)=0;
df_y_2(j)=0;
end

if check_3<tol
df_x_3(j)=0;
df_y_3(j)=0;
end

if check_4<tol
df_x_4(j)=0;
df_y_4(j)=0;
end

% calc average gradient components ----------------------------------------
if nok(j)==4
    df_x(j)=(df_x_1(j)+df_x_2(j)+df_x_3(j)+df_x_4(j))/4;
    df_y(j)=(df_y_1(j)+df_y_2(j)+df_y_3(j)+df_y_4(j))/4;
elseif nok(j)==3
    if DVL_OK(1,j)==0
     df_x(j)=df_x_3(j);
     df_y(j)=df_y_3(j); 
    elseif DVL_OK(2,j)==0
     df_x(j)=df_x_4(j);
     df_y(j)=df_y_4(j); 
    elseif DVL_OK(3,j)==0
     df_x(j)=df_x_1(j);
     df_y(j)=df_y_1(j);     
    elseif DVL_OK(4,j)==0
     df_x(j)=df_x_2(j);
     df_y(j)=df_y_2(j);
    else
     df_x(j)=0;
     df_y(j)=0;   
    end    
else % use previous grads if not OK (or 0)
    df_x(j)=0;%df_x(j-1);
    df_y(j)=0;%df_x(j-1);   
end
% end avg grad comp -------------------------------------------------------

a(j)=r_n(3,j)-df_x(j)*r_n(1,j)-df_y(j)*r_n(2,j); % "avg" alt
a_1(j)=r_n_1(3,j)-df_x_1(j)*r_n_1(1,j)-df_y_1(j)*r_n_1(2,j);
a_2(j)=r_n_2(3,j)-df_x_2(j)*r_n_2(1,j)-df_y_2(j)*r_n_2(2,j);
a_3(j)=r_n_3(3,j)-df_x_3(j)*r_n_3(1,j)-df_y_3(j)*r_n_3(2,j);
a_4(j)=r_n_4(3,j)-df_x_4(j)*r_n_4(1,j)-df_y_4(j)*r_n_4(2,j);

end
% altitude derivatives:

ad=zeros(1,l_data); %scalar altitude rate (best option)
ad_1=zeros(1,l_data); %scalar altitude rate
ad_2=zeros(1,l_data);
ad_3=zeros(1,l_data);
ad_4=zeros(1,l_data);

gradF=zeros(l_data,3);
gradF_1=zeros(l_data,3);
gradF_2=zeros(l_data,3);
gradF_3=zeros(l_data,3);
gradF_4=zeros(l_data,3);

for j=1:l_data
phi_hat=eta(4,j);
theta_hat=eta(5,j);
psi_hat=eta(6,j);

Rx_phi=[1 0 0;
     0 cos(phi_hat) -sin(phi_hat);
     0 sin(phi_hat) cos(phi_hat)];

Ry_theta=[cos(theta_hat) 0 sin(theta_hat);
        0 1 0;
        -sin(theta_hat) 0 cos(theta_hat)];
 
Rz_psi=[cos(psi_hat) -sin(psi_hat) 0;
     sin(psi_hat) cos(psi_hat) 0;
     0 0 1]; 
 
Rnb=Rz_psi*Ry_theta*Rx_phi; 
    
gradF(j,:)=[df_x(j) df_y(j) -1];
gradF_1(j,:)=[df_x_1(j) df_y_1(j) -1];
gradF_2(j,:)=[df_x_2(j) df_y_2(j) -1];
gradF_3(j,:)=[df_x_3(j) df_y_3(j) -1];
gradF_4(j,:)=[df_x_4(j) df_y_4(j) -1];

% gradF_1(j,:)=[n_n_1(1,j) n_n_1(2,j) n_n_1(3,j)];
% gradF_2(j,:)=[n_n_2(1,j) n_n_2(2,j) n_n_2(3,j)];
% gradF_3(j,:)=[n_n_3(1,j) n_n_3(2,j) n_n_3(3,j)];
% gradF_4(j,:)=[n_n_4(1,j) n_n_4(2,j) n_n_4(3,j)];

%v=[EstStates(j,5);EstStates(j,6);EstStates(j,7)]; %v=[u v w]'
v=nu(1:3,j);

ad(j)=gradF(j,:)*Rnb*v;
ad_1(j)=gradF_1(j,:)*Rnb*v;
ad_2(j)=gradF_2(j,:)*Rnb*v;
ad_3(j)=gradF_3(j,:)*Rnb*v;
ad_4(j)=gradF_4(j,:)*Rnb*v;
    
end

%% Model
h=0.15; %time step [s]

PHI=1;
DELTA=h;
GAMMA=-h;

%% Kalman filter

y_a=[a;a_1;a_2;a_3;a_4];
u_a=[ad;ad_1;ad_2;ad_3;ad_4];

n_s=size(y_a,2);

x_hat_a=zeros(5,n_s);
x_hat_avg=zeros(1,n_s);

for j=1:5

y=y_a(j,:);

H=1; 

R=5; 
Q=1;

%u=zeros(n_s,1)';
u=u_a(j,:);

P_bar=zeros(1,n_s+1);
P_bar(1,1)=1;

K=zeros(1,n_s);

P_hat=zeros(1,n_s);
P_hat(1,1)=1;

x_hat=zeros(1,n_s);
x_hat(1,1)=y(1);

x_bar=zeros(1,n_s+1);
x_bar(1,1)=y(1);

I=1;%eye(size(DELTA,1));

for k=1:n_s
% Gain:
    K(1,k)=P_bar(1,k)*H'*[H*P_bar(1,k)*H'+R]^-1;
% Corrector:
    P_hat(1,k)=[I-K(1,k)*H]*P_bar(1,k)*[I-K(1,k)*H]'+K(1,k)*R*K(1,k)';
    x_hat(1,k)=x_bar(1,k)+K(1,k)*[y(k)-H*x_bar(1,k)];
%Predictor:
    x_bar(1,k+1)=PHI*x_hat(1,k)+DELTA*u(k); 
    P_bar(1,k+1)=PHI*P_hat(1,k)*PHI'+GAMMA*Q*GAMMA';
end

x_hat_a(j,:)=x_hat;

end

% for j=1:l_data
%     x_hat_avg(j)=mean(x_hat_a(:,j));
% end

%% prep outputs in struct format:
altitudes.approximated.a=a;
altitudes.approximated.a1=a_1;
altitudes.approximated.a2=a_2;
altitudes.approximated.a3=a_3;
altitudes.approximated.a4=a_4;

altitudes.approximated.ad=ad;
altitudes.approximated.ad1=ad_1;
altitudes.approximated.ad2=ad_2;
altitudes.approximated.ad3=ad_3;
altitudes.approximated.ad4=ad_4;

altitudes.gradF.gradF=gradF;
altitudes.gradF.gradF_1=gradF_1;
altitudes.gradF.gradF_2=gradF_2;
altitudes.gradF.gradF_3=gradF_3;
altitudes.gradF.gradF_4=gradF_4;

altitudes.estimated.a=x_hat_a(1,:);
altitudes.estimated.a1=x_hat_a(2,:);
altitudes.estimated.a2=x_hat_a(3,:);
altitudes.estimated.a3=x_hat_a(4,:);
altitudes.estimated.a4=x_hat_a(5,:);




