% log2mat_sim
% 
% Purpose: 
%           Adapted process of simulations from Joystick Simulation System,
%           based on the work done by Fredrik Dukan. 

%               
% Record of revisions:
% Date:           Programmer:                    Description of change:
%--------------------------------------------------------------------------
% Mai 25, 2013       Paal Lohne                 Changed the original file
%                                               log2mat to be used for
%                                               Joystick Sim System
%
%--------------------------------------------------------------------------
clear;clc;close all

% INPUT:-------------------------------------------------------------------
date_name='20130523*';  % file ID 
% END INPUT----------------------------------------------------------------

path_c=cd;
addpath([path_c,'/Postprofcn']); % locate/call subroutines

folder_asci='Logfiles';      % Data folder with recordings

listing=dir([folder_asci,'/',date_name]);

%% get length of all files-------------------------------------------------
l_tot=0;
for i=1:length(listing)

date_time=listing(i).name(1:14);  % file ID 

file_asci_raw=[date_time,'raw_data']; % force data
asci_path_raw=[path_c,'/',folder_asci,'/',file_asci_raw,'.log'];
[time_t, t_rows, t_cols]=readLogFile_time(asci_path_raw); 

l_tot=l_tot+t_rows;

end

%% get data from log files:------------------------------------------------
l_t=0;
t_count=0;
date_str=cell(l_tot,1);
time.str=cell(l_tot,1);


%--------------------------------------------------------------------------
%% states data in struct format:
% states data -------------------------------------------------------------
states.measured.eta=zeros(6,l_tot);         % Measured eta     
states.desired.eta=zeros(6,l_tot);          % Sperre compass or DVL heading [degs]
 
states.offset.eta=[0 0];  % offset in etan (local coordsystem)

%--------------------------JOYSTICK
joystick.input=zeros(4,l_tot);         % Joystick Commands
joystick.ref=zeros(6,l_tot);            %Joystick Reference Output
%--------------JOYSTICK END-----------------------

%--------------------------------------------------------------------------
%% states data in struct format:
% states data -------------------------------------------------------------
states.measured.nu=zeros(6,l_tot);         % Measured nu 
states.desired.nu=zeros(6,l_tot);          % Desired velocities
%--------------------------------------------------------------------------
%% tau data in struct format:
% tau from controller -----------------------------------------------------
tau.force=zeros(6,l_tot);       % tau in [N/Nm] 
%
%--------------------------------------------------------------------------
%%Acceleration data in struct format:
% tau from controller -----------------------------------------------------
states.measured.nu_dot=zeros(6,l_tot);         % Measured acceleration
states.desired.nu_dot=zeros(6,l_tot);          % Desired acceleration


for i=1:length(listing)

% INPUT:-------------------------------------------------------------------
date_time=listing(i).name(1:14);  % file ID 
% END INPUT----------------------------------------------------------------

% Generate filenames:
file_asci_raw=[date_time,'raw_data']; % position data 
file_asci_desired=[date_time,'desired_data'];   % velocity data
file_asci_joy=[date_time,'joy_data']; % raw data

asci_path_raw=[path_c,'/',folder_asci,'/',file_asci_raw,'.log'];
asci_path_desired=[path_c,'/',folder_asci,'/',file_asci_desired,'.log'];
asci_path_joy=[path_c,'/',folder_asci,'/',file_asci_joy,'.log'];

%% Read asci files for data
[ts1, eta_mes, nu_mes, nu_dot_mes, force]=readLogFile_raw(asci_path_raw);
[ts2, eta_des, nu_des, nu_dot_des]=readLogFile_desired(asci_path_desired); 
[ts3, joy_mes, joy_ref]=readLogFile_joy(asci_path_joy);
%% Create time vectors
l_t=length(ts1);

% convert string time data to MATLAB numeric time data
% Include Meta data:
date=[date_time(7:8),'/',date_time(5:6),'/',date_time(1:4)]; 
format_date='dd/mm/yyyy HH:MM:SS.FFF';
time.UTC='+1'; % some time meta data...

for j = 1:l_t
time.str{j+t_count}=ts1{j};  
date_str{j+t_count}=[date,' ',ts1{j}]; % matlab serial time (days since Jesus was born)
%end

%% Make structs, timeseries and tscollections
%--------------------------------------------------------------------------
%% states data in struct format:
% states data -------------------------------------------------------------
states.measured.eta(:,j+t_count)=eta_mes(:,j);         % Measured eta 
states.desired.eta(:,j+t_count)=eta_des(:,j);          % Sperre compass or DVL heading [degs]
%----JOYSTICK--------------
joystick.input(:,j+t_count)=joy_mes(:,j); 
joystick.ref(:,j+t_count)=joy_ref(:,j); 
%-----JOYSTICK END-----
 
%states.offset.eta=[0 0];  % offset in etan (local coordsystem)
%--------------------------------------------------------------------------
%% states data in struct format:
% states data -------------------------------------------------------------
states.measured.nu(:,j+t_count)=nu_mes(:,j);         % Measured nu 
states.desired.nu(:,j+t_count)=nu_des(:,j);          % Desired velocities

%%%----Acceleration data----------
states.measured.nu_dot(:,j+t_count)=nu_dot_mes(:,j);         % Measured acceleration
states.desired.nu_dot(:,j+t_count)=nu_dot_des(:,j);          % Desired acceleration
%--------------------------------------------------------------------------
%% tau data in struct format:
% tau from controller -----------------------------------------------------
tau.force(:,j+t_count)=force(:,j);       % tau in [N/Nm] 
%--------------------------------------------------------------------------
end

t_count=t_count+l_t;
end

% after time calculations:-------------------------------------------------
time.serial = datenum(date_str,format_date); % matlab serial time (days since Jesus was born)

% make other time formats:
time.vec=datevec(time.serial);        % time in datevec format [yyyy mm dd HH MM SS]

f_vec=[3600,60,1];
time.ssmn=time.vec(:,4:6)*f_vec.';    % time since midnight in seconds
time.ssof=time.ssmn-time.ssmn(1);     % time since start of file in seconds
% end time vectors --------------------------------------------------------

%% use TIMESERIES with the numeric format time data rather than strings.
%% Make timeseries collection
%% use TIMESERIES with the numeric format time data rather than strings.
Eta_mes = timeseries(states.measured.eta',time.serial,'IsTimeFirst',true);
Eta_mes.Name = 'Eta_mes';
Eta_mes.TimeInfo.Units = 'days';
Eta_mes.TimeInfo.StartDate='0-Jan-0000 00:00:00'; % Set start date.
Eta_mes.TimeInfo.Format = 'HH:MM:SS';   % Set format for display on x-axis.

Eta_des = timeseries(states.desired.eta',time.serial,'IsTimeFirst',true);
Eta_des.Name = 'Eta_des';
Eta_des.TimeInfo.Units = 'days';
Eta_des.TimeInfo.StartDate=Eta_mes.TimeInfo.StartDate; % Set start date.
Eta_des.TimeInfo.Format = 'HH:MM:SS';   % Set format for display on x-axis.

%% use TIMESERIES with the numeric format time data rather than strings.
nu_mes = timeseries(states.measured.nu',time.serial,'IsTimeFirst',true);
nu_mes.Name = 'nu_mes';
nu_mes.TimeInfo.Units = 'days';
nu_mes.TimeInfo.StartDate='0-Jan-0000 00:00:00'; % Set start date.
nu_mes.TimeInfo.Format = 'HH:MM:SS';   % Set format for display on x-axis.

nu_des = timeseries(states.desired.nu',time.serial,'IsTimeFirst',true);
nu_des.Name = 'nu_des';
nu_des.TimeInfo.Units = 'days';
nu_des.TimeInfo.StartDate=nu_mes.TimeInfo.StartDate; % Set start date.
nu_des.TimeInfo.Format = 'HH:MM:SS';   % Set format for display on x-axis.

%% use TIMESERIES with the numeric format time data rather than strings.
nu_dot_mes = timeseries(states.measured.nu_dot',time.serial,'IsTimeFirst',true);
nu_dot_mes.Name = 'nu_dot_mes';
nu_dot_mes.TimeInfo.Units = 'days';
nu_dot_mes.TimeInfo.StartDate='0-Jan-0000 00:00:00'; % Set start date.
nu_dot_mes.TimeInfo.Format = 'HH:MM:SS';   % Set format for display on x-axis.

nu_dot_des = timeseries(states.desired.nu_dot',time.serial,'IsTimeFirst',true);
nu_dot_des.Name = 'nu_dot_des';
nu_dot_des.TimeInfo.Units = 'days';
nu_dot_des.TimeInfo.StartDate=nu_dot_mes.TimeInfo.StartDate; % Set start date.
nu_dot_des.TimeInfo.Format = 'HH:MM:SS';   % Set format for display on x-axis.

%% Make timeseries collection 
% of states data
States = tscollection({Eta_mes Eta_des nu_mes nu_des nu_dot_mes nu_dot_des},'name','States');

%-------------------JOYSTICK DATA-------------------
%% use TIMESERIES with the numeric format time data rather than strings.

%% Make timeseries collection 
% of states data
Joy_mes = timeseries(joystick.input',time.serial,'IsTimeFirst',true);
Joy_mes.Name = 'Joy_mes';
Joy_mes.TimeInfo.Units = 'days';
Joy_mes.TimeInfo.StartDate='0-Jan-0000 00:00:00'; % Set start date.
Joy_mes.TimeInfo.Format = 'HH:MM:SS';   % Set format for display on x-axis.

Joy_ref = timeseries(joystick.ref',time.serial,'IsTimeFirst',true);
Joy_ref.Name = 'Joy_ref';
Joy_ref.TimeInfo.Units = 'days';
Joy_ref.TimeInfo.StartDate=Joy_mes.TimeInfo.StartDate; % Set start date.
Joy_ref.TimeInfo.Format = 'HH:MM:SS';   % Set format for display on x-ax

Joystick = tscollection({Joy_mes Joy_ref},'name','Joystick');
%----------------Joystick DATA END----------


%% use TIMESERIES with the numeric format time data rather than strings.
force = timeseries(tau.force',time.serial,'IsTimeFirst',true);
force.Name = 'Force';
force.TimeInfo.Units = 'days';
force.TimeInfo.StartDate='0-Jan-0000 00:00:00'; % Set start date.
force.TimeInfo.Format = 'HH:MM:SS';   % Set format for display on x-axis.

%% Make timeseries collection
% of force data (tau)
Tau = tscollection({force},'name','Tau');

%% Save data to .mat file:

folder_mat='Logfiles_mat';
mat_path=[path_c,'/',folder_mat,'/',date_time,'day.mat']; % mat folder

save(mat_path, 'time', 'states', 'States', 'tau', 'Tau', 'joystick', 'Joystick');
%clearvars -except time raw Raw states States tau Tau
%load(mat_path);

clearvars -except time states States tau Tau joystick Joystick
