clc;clear;close all;
% plot script for post processing of ROV data
% 
% Purpose: 
%           Plotting and post-processing data
%
% Subroutines:  1) 
%               2)
%               
%                  
% Comments: some of the functions may require the gnc package to be
%           installed (www.marinecontrol.org). 
%
% Record of revisions:
% Date:        Programmer:                    Description of change:
%--------------------------------------------------------------------------
% 14 Nov 12    Fredrik Dukan                  org.
% 20 Nov 12    FD                             new log style with structs
% 
%--------------------------------------------------------------------------
tic
% INPUT:-------------------------------------------------------------------
date_time='20130510';  % file ID 
% END INPUT----------------------------------------------------------------

path_c=cd;
addpath([path_c,'\Postprofcn']);
folder_mat='Logfiles_mat';
mat_path=[path_c,'\',folder_mat,'\',date_time,'.mat'];

load(mat_path);
%% Select time interval and type
start_t='13:20:00';
end_t='13:25:00';

sflag=1;
tflag=0;        % selct which time to use in plots
plot_select=0;

% make other time formats:
time_vec=datevec(datenum([start_t; end_t],'HH:MM:SS'));        % time in datevec format [yyyy mm dd HH MM SS]

f_vec=[3600,60,1];
time_ssmn=time_vec(:,4:6)*f_vec.';    % time since midnight in seconds
time_ssof=time_ssmn-time.ssmn(1);     % time since start of file in seconds

[start,stop] = partial_data(time_ssof(1),time_ssof(2),sflag,time.ssof);

if tflag==0     % use serial date
time_temp=time.serial(start:stop);
elseif tflag==1 % use seconds since midnight
time_temp=time.ssmn(start:stop);
else            % use seconds since start of file
time_temp=time.ssof(start:stop);
end

%% RUN post processing:
% find key points (start, stop points)
clear start_s stop_s

pause=1;
count_s=1; % counter for start points
count_e=1; % counter for stop/end points

x_temp=states.desired.eta(2,start:stop);
l_t=length(time_temp);

for i=1:(l_t-1)
    
   if x_temp(i+1)==x_temp(i)
       if pause==0
        stop_s.index(count_e)=i;
        stop_s.time{count_e}=datestr(time_temp(i),'HH:MM:SS');
        pause=1;
        count_e=count_e+1;
       end    
   elseif x_temp(i+1)~=x_temp(i)
       if pause==1
        start_s.index(count_s)=i;
        start_s.time{count_s}=datestr(time_temp(i),'HH:MM:SS');
        pause=0;
        count_s=count_s+1;
       end
   end
    
end

%%
%DVL altitude estimation:
%call sub routine dvl_kf for post processing of dvl data.
%[altitudes]=dvl_kf_beta(raw,states);
[altitudes,nok]=dvl_kf_beta(raw.DVL.altitudes(:,start:stop),states.estimated.eta(:,start:stop),states.estimated.nu(:,start:stop),raw.DVL.offset,raw.DVL.alignment);

[alt2,gradFss,n_nss,adss,abcss,r_p]=dvl_approx_beta(raw.DVL.altitudes(:,start:stop),states.estimated.eta(:,start:stop),states.estimated.nu(:,start:stop),raw.DVL.alignment);
r_ps=[r_p(:,:,1) r_p(:,:,2) r_p(:,:,3) r_p(:,:,4)]';

%%
x_dvl=zeros(4*l_t,1);
y_dvl=zeros(4*l_t,1);
z_dvl=zeros(4*l_t,1);

for i=1:1:l_t
   
x_dvl(4*i-3)=r_p(1,i,1); 
x_dvl(4*i-2)=r_p(1,i,2);    
x_dvl(4*i-1)=r_p(1,i,3);   
x_dvl(4*i)=r_p(1,i,4);

y_dvl(4*i-3)=r_p(2,i,1); 
y_dvl(4*i-2)=r_p(2,i,2);    
y_dvl(4*i-1)=r_p(2,i,3);   
y_dvl(4*i-0)=r_p(2,i,4);

z_dvl(4*i-3)=r_p(3,i,1); 
z_dvl(4*i-2)=r_p(3,i,2);    
z_dvl(4*i-1)=r_p(3,i,3);   
z_dvl(4*i-0)=r_p(3,i,4);

end

postpro.altitudes=altitudes; %get into struct 

%w_depth=states.estimated.eta(3,start:stop)+altitudes.estimated.a;
w_depth=states.estimated.eta(3,:)+states.estimated.alt;

% Accumulated horizontal distance:
l_t=stop-start+1;
s_rov=zeros(l_t,1);

for i=2:l_t
s_rov(i)=s_rov(i-1)+((states.estimated.eta(1,start+i)-states.estimated.eta(1,start+i-1))^2+(states.estimated.eta(2,start+i)-states.estimated.eta(2,start+i-1))^2)^(1/2);
end

%--------------------------------------------------------------------------
% MTi
% run attitude estimates off-line:-----------------------------------------
%[THETA_hat,THETA_m,G_BIAS]=MTi_ecf(MT_SINGLE,time.ssof); %Get attitude estimates from post processing
%% ROV positions:-----------------------------------------------------------
figure
ax0(1)=subplot(3,1,1);
plot(time_temp,states.measured.eta(1,start:stop),'b',time_temp,states.estimated.eta(1,start:stop),'r',time_temp,states.desired.eta(1,start:stop),'g')
title('North')
xlabel('t [s]')
ylabel('N [m]')
hleg=legend('$x_m$','$\hat{x}$','$x_d$');
set(hleg,'interpreter','latex')
grid on

ax0(2)=subplot(3,1,2);
plot(time_temp,states.measured.eta(2,start:stop),'b',time_temp,states.estimated.eta(2,start:stop),'r',time_temp,states.desired.eta(2,start:stop),'g')
title('East')
xlabel('t [s]')
ylabel('E [m]')
hleg=legend('$y_m$','$\hat{y}$','$y_d$');
set(hleg,'interpreter','latex')
grid on

ax0(3)=subplot(3,1,3);
plot(time_temp,states.measured.eta(3,start:stop),'b',time_temp,states.estimated.eta(3,start:stop),'r',time_temp,states.desired.eta(3,start:stop),'g')
title('Depth')
xlabel('t [s]')
ylabel('z [m]')
hleg=legend('$z_m$','$\hat{z}$','$z_d$');
set(hleg,'interpreter','latex')
grid on

linkaxes(ax0,'x')
if tflag==0
dynamicDateTicks(ax0, 'linked')
end

% Depth reference for altitude control, estimated depth and altitude:------
figure
ax6(1)=subplot(2,1,1);
plot(time_temp,states.desired.alt(start:stop),'-r',time_temp,states.estimated.alt(start:stop),'-k')
title('ROV Altitude')
xlabel('t [s]')
ylabel('a [m]')
hleg=legend('$a^{CO}_d$','$\hat{a}^{CO}_{avg}$');
set(hleg,'interpreter','latex')
%ylim([4.5 6])
grid on

ax6(2)=subplot(2,1,2);
plot(time_temp,states.desired.eta(3,start:stop),'-r',time_temp,states.estimated.eta(3,start:stop),'-k')
title('ROV Depth')
xlabel('t [s]')
ylabel('z [m]')
hleg=legend('$z_d$','$\hat{z}$');
set(hleg,'interpreter','latex')
%ylim([78 87])
grid on

linkaxes(ax6,'x')
if tflag==0
dynamicDateTicks(ax6, 'linked')
end

% %% Test of comparison plot
% % Depth reference for altitude control, estimated depth and altitude:------
% %Down Hill
% 
% %plot_id=start+start_s.index(1):start+stop_s.index(1);
% %t_span=start_s.index(1):stop_s.index(1); %
% %t_span=start_s.index(1)-70:start_s.index(1)+850; %
% start_id=start+start_s.index;
% stop_id=start+stop_s.index;
% 
% p_span=zeros(length(start_s.index),921);
% t_span=zeros(length(start_s.index),921);
% 
% for j=1:length(start_s.index)
% p_span(j,:)=start_id(j)-70:start_id(j)+850;
% t_span(j,:)=start_s.index(j)-70:start_s.index(j)+850; %
% end
% 
% figure
% ax_7(1)=subplot(2,1,1);
% plot(states.desired.eta(2,p_span(1,:)),w_depth(p_span(1,:)),'k',states.desired.eta(2,p_span(3,:)),w_depth(p_span(3,:)),'b',states.desired.eta(2,p_span(5,:)),w_depth(p_span(5,:)),'r')
% title('Seafloor Profile, Downhill')
% xlabel('E [m]')
% ylabel('w_d [m]')
% hleg=legend('$a_{80}$','$a_{85}$','$a_{87}$');
% set(hleg,'interpreter','latex')
% grid on
% set(gca,'YDir','reverse')
% 
% ax_7(2)=subplot(2,1,2);
% plot(states.estimated.eta(2,p_span(1,:)),states.estimated.alt(p_span(1,:)),'k',states.estimated.eta(2,p_span(3,:)),states.estimated.alt(p_span(3,:)),'b',states.estimated.eta(2,p_span(5,:)),states.estimated.alt(p_span(5,:)),'r')
% title('ROV Altitude, Downhill')
% xlabel('E [m]')
% ylabel('a [m]')
% hleg=legend('$a_{80}$','$a_{85}$','$a_{87}$');
% set(hleg,'interpreter','latex')
% grid on
% 
% linkaxes([ax_7(1) ax_7(2)],'x');  

% figure
% ax7a(1)=subplot(2,1,1);
% plot(time_temp(t_span),states.desired.alt(p_span(1,:)),'g',time_temp(t_span),states.estimated.alt(p_span(1,:)),'k',time_temp(t_span),states.estimated.alt(p_span(28,:)),'b',time_temp(t_span),states.estimated.alt(p_span(75,:)),'r',time_temp(t_span),states.estimated.alt(p_span(77,:)),'y')
% title('ROV Altitude, Downhill')
% xlabel('t [s]')
% ylabel('a [m]')
% hleg=legend('$a^{CO}_d$','$\hat{a4}^{CO}_{avg}$','$\hat{a1}^{CO}_{avg}$','$\hat{a2}^{CO}_{avg}$','$\hat{a3}^{CO}_{avg}$');
% set(hleg,'interpreter','latex')
% grid on
% 
% ax7a(2)=subplot(2,1,2);
% plot(time_temp(t_span),states.desired.eta(3,p_span(1,:)),'--k',time_temp(t_span),states.estimated.eta(3,p_span(1,:)),'k',time_temp(t_span),states.desired.eta(3,p_span(28,:)),'--b',time_temp(t_span),states.estimated.eta(3,p_span(28,:)),'b',time_temp(t_span),states.desired.eta(3,p_span(75,:)),'--r',time_temp(t_span),states.estimated.eta(3,p_span(75,:)),'r',time_temp(t_span),states.desired.eta(3,p_span(77,:)),'--y',time_temp(t_span),states.estimated.eta(3,p_span(77,:)),'y')
% title('ROV Depth, Downhill')
% xlabel('t [s]')
% ylabel('z [m]')
% hleg=legend('$z_d$','$\hat{z}$');
% set(hleg,'interpreter','latex')
% grid on
% 
% linkaxes(ax7a,'x')
% if tflag==0
% dynamicDateTicks(ax7a, 'linked')
% end




%% DVL altitudes-----------------------------------------------------------
% DVL and est altitudes in mission:----------------------------------------
figure
ax(1)=subplot(2,1,1);
plot(time_temp,raw.DVL.altitudes(1,start:stop),'-b',time_temp,raw.DVL.altitudes(2,start:stop),'--r',time_temp,raw.DVL.altitudes(3,start:stop),':k',time_temp,raw.DVL.altitudes(4,start:stop),'-.g') 
title('DVL Altitudes')
xlabel('t [s]')
ylabel('a [m]')
hleg=legend('$a^{DVL}_1$','$a^{DVL}_2$','$a^{DVL}_3$','$a^{DVL}_4$');
set(hleg,'interpreter','latex')
grid on

ax(2)=subplot(2,1,2);
plot(time_temp,altitudes.estimated.a,'k',time_temp,altitudes.estimated.a1,'-b',time_temp,altitudes.estimated.a2,'--r',time_temp,altitudes.estimated.a3,':m',time_temp,altitudes.estimated.a4,'-.g')
title('ROV Estimated Altitudes')
xlabel('t [s]')
ylabel('a [m]')
hleg=legend('$\hat{a}^{CO}$','$\hat{a}^{CO}_1$','$\hat{a}^{CO}_2$','$\hat{a}^{CO}_3$','$\hat{a}^{CO}_4$');
set(hleg,'interpreter','latex')
grid on

linkaxes(ax,'x')
if tflag==0
dynamicDateTicks(ax, 'linked')
end

if plot_select==3
% approximated and rates (input to altitude observer):---------------------
figure
ax1(1)=subplot(2,1,1);
plot(time_temp,altitudes.approximated.a1,'-b',time_temp,altitudes.approximated.a2,'--r',time_temp,altitudes.approximated.a3,':m',time_temp,altitudes.approximated.a4,'-.g') 
title('Approximated Altitudes from DVL Measurements')
xlabel('t [s]')
ylabel('a [m]')
hleg=legend('$a^{DVL}_1$','$a^{DVL}_2$','$a^{DVL}_3$','$a^{DVL}_4$');
set(hleg,'interpreter','latex')
grid on

ax1(2)=subplot(2,1,2);
plot(time_temp,altitudes.approximated.ad1,'-b',time_temp,altitudes.approximated.ad2,'--r',time_temp,altitudes.approximated.ad3,':m',time_temp,altitudes.approximated.ad4,'-.g')
title('Approximated Altitude Rate of Change')
xlabel('t [s]')
ylabel('$\dot{a}$ [m/s]', 'interpreter', 'latex')
hleg=legend('$\hat{a}^{CO}_1$','$\hat{a}^{CO}_2$','$\hat{a}^{CO}_3$','$\hat{a}^{CO}_4$','$\hat{a}^{CO}_{avg}$');
set(hleg,'interpreter','latex')
grid on

linkaxes(ax1,'x')
if tflag==0
dynamicDateTicks(ax1, 'linked')
end

end %plot select
figure
ax1b(1)=subplot(2,1,1);
plot(time_temp,altitudes.approximated.a,'b') 
title('Approximated Altitudes from DVL Measurements')
xlabel('t [s]')
ylabel('a [m]')
hleg=legend('$a^{DVL}$');
set(hleg,'interpreter','latex')
grid on

ax1b(2)=subplot(2,1,2);
plot(time_temp,altitudes.approximated.ad,'b')
title('Approximated Altitude Rate of Change')
xlabel('t [s]')
ylabel('$\dot{a}$ [m/s]', 'interpreter', 'latex')
hleg=legend('$\dot{a}^{CO}$');
set(hleg,'interpreter','latex')
grid on

linkaxes(ax1b,'x')
if tflag==0
dynamicDateTicks(ax1b, 'linked')
end

if plot_select==3
% approximated and estimated ----------------------------------------------
figure
ax2(1)=subplot(2,2,1);
plot(time_temp,altitudes.approximated.a1,'b',time_temp,altitudes.estimated.a1,'r') 
xlabel('t [s]')
ylabel('a [m]')
hleg=legend('$a^{DVL}_1$','$\hat{a}^{DVL}_1$');
set(hleg,'interpreter','latex')
grid on

ax2(2)=subplot(2,2,2);
plot(time_temp,altitudes.approximated.a2,'b',time_temp,altitudes.estimated.a2,'r') 
%title('Approximated vs Estimated Altitudes from DVL Measurements')
xlabel('t [s]')
ylabel('a [m]')
hleg=legend('$a^{DVL}_2$','$\hat{a}^{DVL}_2$');
set(hleg,'interpreter','latex')
grid on

ax2(3)=subplot(2,2,3);
plot(time_temp,altitudes.approximated.a3,'b',time_temp,altitudes.estimated.a3,'r') 
%title('Approximated vs Estimated Altitudes from DVL Measurements')
xlabel('t [s]')
ylabel('a [m]')
hleg=legend('$a^{DVL}_3$','$\hat{a}^{DVL}_3$');
set(hleg,'interpreter','latex')
grid on

ax2(4)=subplot(2,2,4);
plot(time_temp,altitudes.approximated.a4,'b',time_temp,altitudes.estimated.a4,'r') 
%title('Approximated vs Estimated Altitudes from DVL Measurements')
xlabel('t [s]')
ylabel('a [m]')
hleg=legend('$a^{DVL}_4$','$\hat{a}^{DVL}_4$');
set(hleg,'interpreter','latex')
grid on

title('Approximated vs Estimated Altitudes from DVL Measurements')

linkaxes(ax2,'x')
if tflag==0
dynamicDateTicks(ax2, 'linked')
end

end %plot_select

%% approximated and estimated ----------------------------------------------
figure
ax3(1)=subplot(2,1,1);
plot(time_temp,altitudes.approximated.a,'b',time_temp,altitudes.estimated.a,'r') 
title('Approximated vs Estimated Altitudes from DVL Measurements, "error handling"')
xlabel('t [s]')
ylabel('a [m]')
hleg=legend('$a^{DVL}$','$\hat{a}^{DVL}$');
set(hleg,'interpreter','latex')
grid on

ax3(2)=subplot(2,1,2);
plot(time_temp,altitudes.approximated.a1,'b',time_temp,altitudes.estimated.a1,'r') 
title('Approximated vs Estimated Altitudes from DVL Measurements, "raw"')
xlabel('t [s]')
ylabel('a [m]')
hleg=legend('$a^{DVL}_1$','$\hat{a}^{DVL}_1$');
set(hleg,'interpreter','latex')
grid on

linkaxes(ax3,'x')
if tflag==0
dynamicDateTicks(ax3, 'linked')
end

% Gradient vector, grad F=[dF/dx,dF/dy,dF/dz] or grad F=[df/dx,df/dy,-1]:--
figure
%subplot(2,1,1)
plot(time_temp,altitudes.gradF.gradF(:,1),'b',time_temp,altitudes.gradF.gradF(:,2),'r',time_temp,altitudes.gradF.gradF(:,3),'k') 
title('Gradients')
xlabel('t [s]')
ylabel('grad F [-]')
legend('df/dx','df/dy','-1')
grid on

if tflag==0
dynamicDateTicks
end

%% Terrain profile from estimated depth and altitude:------
figure
ax6b(1)=subplot(1,1,1);
plot(time_temp,(altitudes.estimated.a+states.estimated.eta(3,start:stop)),time_temp,states.estimated.eta(3,start:stop),'-r')
title('ROV Altitude')
xlabel('t [s]')
ylabel('z [m]')
hleg=legend('$z_{terrain}$','$\hat{z}_{ROV}$');%,'$\hat{a}^{DVL}_{avg}$');
set(hleg,'interpreter','latex')
set(gca,'YDir','reverse')
%ylim([4.5 6])
grid on

linkaxes(ax6b,'x')
if tflag==0
dynamicDateTicks(ax6b, 'linked')
end

figure
subplot(2,1,1)
plot(states.estimated.eta(1,start:stop),(altitudes.estimated.a+states.estimated.eta(3,start:stop)),states.estimated.eta(1,start:stop),states.estimated.eta(3,start:stop),'-r')
title('ROV Terrain Following')
xlabel('N [m]')
ylabel('z [m]')
hleg=legend('$z_{terrain}$','$\hat{z}_{ROV}$');%,'$\hat{a}^{DVL}_{avg}$');
set(hleg,'interpreter','latex')
set(gca,'YDir','reverse')
%ylim([4.5 6])
axis equal
grid on

subplot(2,1,2)
plot(states.estimated.eta(2,start:stop),(altitudes.estimated.a+states.estimated.eta(3,start:stop)),states.estimated.eta(2,start:stop),states.estimated.eta(3,start:stop),'-r')
title('ROV Terrain Following')
xlabel('E [m]')
ylabel('z [m]')
hleg=legend('$z_{terrain}$','$\hat{z}_{ROV}$');%,'$\hat{a}^{DVL}_{avg}$');
set(hleg,'interpreter','latex')
set(gca,'YDir','reverse')
%ylim([4.5 6])
axis equal
grid on

figure
plot(s_rov,(altitudes.estimated.a+states.estimated.eta(3,start:stop)),s_rov,states.estimated.eta(3,start:stop),'-r')
title('ROV Terrain Following Profile')
xlabel('distance [m]')
ylabel('z [m]')
hleg=legend('$z_{terrain}$','$\hat{z}_{ROV}$');%,'$\hat{a}^{DVL}_{avg}$');
set(hleg,'interpreter','latex')
set(gca,'YDir','reverse')
%ylim([4.5 6])
axis equal
grid on


