clc;clear all;close all;
% plot script for ROV data
% 
% Purpose: 
%           Plotting of onboard processed data and states
%
% Subroutines:  1) 
%               2)
%               
%                  
% Comments: some of the functions may require the gnc package to be
%           installed (www.marinecontrol.org). 
%
% Record of revisions:
% Date:        Programmer:                    Description of change:
%--------------------------------------------------------------------------
% 14 Nov 12    Fredrik Dukan                  org.
% 20 Nov 12    FD                             new log style with structs
% 
%--------------------------------------------------------------------------
tic
% INPUT:-------------------------------------------------------------------
date_time='20130523';  % file ID 
start_t='11:31:25';
end_t='11:32:20';

sflag=1;
tflag=0;        % selct which time to use in plots
% END INPUT----------------------------------------------------------------

path_c=cd;
addpath([path_c,'/Postprofcn']);
folder_mat='Logfiles_mat';
mat_path=[path_c,'/',folder_mat,'/',date_time,'.mat'];

load(mat_path);

%% Calculate time formats

% make other time formats:
time_vec=datevec(datenum([start_t; end_t],'HH:MM:SS'));        % time in datevec format [yyyy mm dd HH MM SS]

f_vec=[3600,60,1];
time_ssmn=time_vec(:,4:6)*f_vec.';    % time since midnight in seconds
time_ssof=time_ssmn-time.ssmn(1);     % time since start of file in seconds

[start,stop] = partial_data(time_ssof(1),time_ssof(2),sflag,time.ssof);

tflag=0;        % selct which time to use in plots
if tflag==0     % use serial date
time_temp=time.serial(start:stop);
elseif tflag==1 % use seconds since midnight
time_temp=time.ssmn(start:stop);
else            % use seconds since start of file
time_temp=time.ssof(start:stop);
end

%temp = floor((stop+start)/2);
temp = start;

for i = start:stop
    count=1;
    stat1(count) = states.measured.eta(1,temp);
    %stat2(count) = states.estimated.eta(1,temp);
    stat3(count) = states.desired.eta(1,temp);
    stat4(count) = states.measured.eta(2,temp);
    %stat5(count) = states.estimated.eta(2,temp);
    stat6(count) = states.desired.eta(2,temp);
    count = count + 1;
end
%% ROV positions:-----------------------------------------------------------
figure
ax1(1)=subplot(3,1,1);
plot(time_temp,states.measured.eta(1,start:stop)-stat1,'b',time_temp,states.desired.eta(1,start:stop)-stat3,'g')
title('North','FontSize',20)
xlabel('t [s]','FontSize',20)
ylabel('N [m]','FontSize',20)
hleg=legend('$x_m$','$x_d$');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on

ax1(2)=subplot(3,1,2);
plot(time_temp,states.measured.eta(2,start:stop)-stat4,'b',time_temp,states.desired.eta(2,start:stop)-stat6,'g')
title('East','FontSize',20)
xlabel('t [s]','FontSize',20)
ylabel('E [m]','FontSize',20)
hleg=legend('$y_m$','$y_d$');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on

ax1(3)=subplot(3,1,3);
plot(time_temp,states.measured.eta(3,start:stop),'b',time_temp,states.desired.eta(3,start:stop),'g')
title('Depth','FontSize',20)
xlabel('t [s]','FontSize',20)
ylabel('z [m]','FontSize',20)
hleg=legend('$z_m$','$z_d$');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on

linkaxes(ax1,'x')
if tflag==0
dynamicDateTicks(ax1, 'linked')
axis auto;
end

%%===== ROV attitudes:-----------------------------------------------------------
% figure
% ax2(1)=subplot(3,1,1);
% plot(time_temp,states.measured.eta(4,start:stop)*180/pi,'b',time_temp,states.estimated.eta(4,start:stop)*180/pi,'r',time_temp,states.desired.eta(4,start:stop)*180/pi,'g')
% title('Roll')
% xlabel('t [s]')
% ylabel('\phi [deg]')
% hleg=legend('$\phi_m$','$\hat{\phi}$','$\phi_d$');
% set(hleg,'interpreter','latex')
% grid on
% 
% ax2(2)=subplot(3,1,2);
% plot(time_temp,states.measured.eta(5,start:stop)*180/pi,'b',time_temp,states.estimated.eta(5,start:stop)*180/pi,'r',time_temp,states.desired.eta(5,start:stop)*180/pi,'g')
% title('Pitch')
% xlabel('t [s]')
% ylabel('\theta [deg]')
% hleg=legend('$\theta_m$','$\hat{\theta}$','$\theta_d$');
% set(hleg,'interpreter','latex')
% grid on
% 
% ax2(3)=subplot(3,1,3);
% plot(time_temp,states.measured.eta(6,start:stop)*180/pi,'b',time_temp,states.estimated.eta(6,start:stop)*180/pi,'r',time_temp,states.desired.eta(6,start:stop)*180/pi,'g')
% title('Yaw')
% xlabel('t [s]')
% ylabel('\psi [deg]')
% hleg=legend('$\psi_m$','$\hat{\psi}$','$\psi_d$');
% set(hleg,'interpreter','latex')
% grid on
% 
% linkaxes(ax2,'x')
% if tflag==0
% dynamicDateTicks(ax2, 'linked')
% end

% ROV velocities: ---------------------------------------------------------
figure
ax3(1)=subplot(3,1,1);
plot(time_temp,states.measured.nu(1,start:stop),'b',time_temp,states.desired.nu(1,start:stop),'g')
title('Surge Velocity','FontSize',20)
xlabel('t [s]','FontSize',20)
ylabel('u [m/s]','FontSize',20)
hleg=legend('$u_m$','$u_d$');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on

ax3(2)=subplot(3,1,2);
plot(time_temp,states.measured.nu(2,start:stop),'b',time_temp,states.desired.nu(2,start:stop),'g')
title('Sway Velocity','FontSize',20)
xlabel('t [s]','FontSize',20)
ylabel('v [m/s]','FontSize',20)
hleg=legend('$v_m$','$v_d$');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on

ax3(3)=subplot(3,1,3);
plot(time_temp,states.measured.nu(6,start:stop),'b',time_temp,states.desired.nu(6,start:stop),'g')
title('Yaw Rate','FontSize',20)
xlabel('t [s]','FontSize',20)
ylabel('r [deg/s]','FontSize',20)
hleg=legend('$r_m$','$r_d$');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on

linkaxes(ax3,'x')
if tflag==0
dynamicDateTicks(ax3, 'linked')
axis auto;
end


% Angular velocities ------------------------------------------------------
figure
ax4(1)=subplot(3,1,1);
plot(time_temp,states.measured.nu(4,start:stop),'b',time_temp,states.desired.nu(4,start:stop),'g')
title('p','FontSize',20)
xlabel('t [s]','FontSize',20)
ylabel('p [deg/s]','FontSize',20)
hleg=legend('$p_m$','$p_d$');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on

ax4(2)=subplot(3,1,2);
plot(time_temp,states.measured.nu(5,start:stop),'b',time_temp,states.desired.nu(5,start:stop),'g')
title('q','FontSize',20)
xlabel('t [s]','FontSize',20)
ylabel('q [deg/s]','FontSize',20)
hleg=legend('$q_m$','$q_d$');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on

ax4(3)=subplot(3,1,3);
plot(time_temp,states.measured.nu(3,start:stop),'b',time_temp,states.desired.nu(3,start:stop),'g')
title('Heave','FontSize',20)
xlabel('t [s]','FontSize',20)
ylabel('w [m/s]','FontSize',20)
hleg=legend('$w_m$', '$w_d$');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on

linkaxes(ax4,'x')
if tflag==0
dynamicDateTicks(ax4, 'linked')
axis auto;
end

%% DVL altitudes-----------------------------------------------------------
% DVL and est altitudes in mission:----------------------------------------
% figure
% ax5(1)=subplot(2,1,1);
% plot(time_temp,raw.DVL.altitudes(1,start:stop),'-b',time_temp,raw.DVL.altitudes(2,start:stop),'--r',time_temp,raw.DVL.altitudes(3,start:stop),':k',time_temp,raw.DVL.altitudes(4,start:stop),'-.g') % map DVL no to quad no: 1->2, 2->4, 3->1, 4->3: (blue 600kHz DVL)
% title('DVL Altitudes')
% xlabel('t [s]')
% ylabel('a [m]')
% hleg=legend('$a^{DVL}_1$','$a^{DVL}_2$','$a^{DVL}_3$','$a^{DVL}_4$');
% set(hleg,'interpreter','latex')
% %ylim([7 19])
% grid on
% 
% ax5(2)=subplot(2,1,2);
% plot(time_temp,states.measured.alt(start:stop),'-b',time_temp,states.estimated.alt(start:stop),'r',time_temp,states.desired.alt(start:stop),'g')
% title('ROV Altitudes')
% xlabel('t [s]')
% ylabel('a [m]')
% hleg=legend('${a}^{CO}_{app}$','$\hat{a}^{CO}$','${a}^{CO}_d$');
% set(hleg,'interpreter','latex')
% %ylim([7 19])
% grid on
% 
% linkaxes(ax5,'x')
% if tflag==0
% dynamicDateTicks(ax5, 'linked')
% end

% Depth reference for altitude control, estimated depth and altitude:------
% figure
% ax6(1)=subplot(2,1,1);
% plot(time_temp,states.desired.alt(start:stop),'-r',time_temp,states.estimated.alt(start:stop),'-k')
% title('ROV Altitude')
% xlabel('t [s]')
% ylabel('a [m]')
% hleg=legend('$a^{CO}_d$','$\hat{a}^{CO}_{avg}$');
% set(hleg,'interpreter','latex')
% %ylim([4.5 6])
% grid on
% 
% ax6(2)=subplot(2,1,2);
% plot(time_temp,states.desired.eta(3,start:stop),'-r',time_temp,states.estimated.eta(3,start:stop),'-k')
% title('ROV Depth')
% xlabel('t [s]')
% ylabel('z [m]')
% hleg=legend('$z_d$','$\hat{z}$');
% set(hleg,'interpreter','latex')
% %ylim([78 87])
% grid on
% 
% linkaxes(ax6,'x')
% if tflag==0
% dynamicDateTicks(ax6, 'linked')
% end

%% Control forces (BODY)---------------------------------------------------

% Commanded control forces and corresponding thruster rpm:-----------------
figure
plot(time_temp,tau.force(:,start:stop));
title('Commanded Control Forces','FontSize',20)
xlabel('t [s]','FontSize',20)
ylabel('Force [N/Nm]','FontSize',20)
legend('X [N]', 'Y [N]', 'Z [N]','K [Nm]', 'M [Nm]', 'N [Nm]','FontSize',20)
set(gca,'FontSize',14)
grid on


if tflag==0
dynamicDateTicks()
axis auto;
end

%% ROV plot Fredrik
% snap shot
% l = 2.44; 				% length (m)
% b = 1.5;				% breadth (m)
% h = 1.8;                % height (m)
l = 1.44; 				% length (m)
b = 0.82;				% breadth (m)
h = 0.8;                % height (m)

l_data=length(time_temp);

figure
hold on 
plot(states.measured.eta(2,start:stop)-stat4,states.measured.eta(1,start:stop)-stat1,'b',states.desired.eta(2,start:stop)-stat6,states.desired.eta(1,start:stop)-stat3,'g')
%plot(states.estimated.eta(2,start:stop),states.estimated.eta(1,start:stop),'b',states.desired.eta(2,start:stop),states.desired.eta(1,start:stop),'g')
for i = start:75:stop
    
    Re  = [cos(states.measured.eta(6,i)) -sin(states.measured.eta(6,i)) 0; sin(states.measured.eta(6,i)) cos(states.measured.eta(6,i)) 0; 0 0 1];
    rov = Re*[l/2 0.9*l/2 0.5*l/2 -l/2 -l/2 0.5*l/2 0.9*l/2 l/2;
             0 b/2 b/2 b/2 -b/2 -b/2 -b/2 0;
             h/2 0.9*h/2 0.5*h/2 -h/2 -h/2 0.5*h/2 0.9*h/2 h/2]; 
    plot3((states.measured.eta(2,i)-states.measured.eta(2,temp))+rov(2,:),(states.measured.eta(1,i)-states.measured.eta(1,temp))+rov(1,:),(states.measured.eta(3,i)-states.measured.eta(3,temp))+rov(3,:),'b');

end

axis equal
xlabel('East Position [m]','FontSize',20)
ylabel('North Position [m]','FontSize',20)
title('ROV Trace','FontSize',20)
hleg=legend('${x}_m,{y}_m,{\psi}_m$','${x}_d,{y}_d$');
%hleg=legend('$\hat{x},\hat{y},\hat{\psi}$','${x}_d,{y}_d$');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on
%%
% figure
% ax10(1)=subplot(2,1,1);
% plot(time_temp,states.measured.nu(3,start:stop),'b',time_temp,states.estimated.nu(3,start:stop),'r',time_temp,states.desired.nu(3,start:stop),'g')
% title('w')
% xlabel('t [s]')
% ylabel('w [m/s]')
% hleg=legend('$w_m$','$\hat{w}$','$w_d$');
% set(hleg,'interpreter','latex')
% grid on
% 
% ax10(2)=subplot(2,1,2);
% plot(time_temp,states.measured.eta(3,start:stop),'b',time_temp,states.estimated.eta(3,start:stop),'r',time_temp,states.desired.eta(3,start:stop),'g')
% title('Depth')
% xlabel('t [s]')
% ylabel('z [m]')
% hleg=legend('$z_m$','$\hat{z}$','$z_d$');
% set(hleg,'interpreter','latex')
% grid on
% 
% linkaxes(ax10,'x')
% if tflag==0
% dynamicDateTicks(ax10, 'linked')
% end

% Joystick References: ---------------------------------------------------------
figure
ax11(1)=subplot(2,1,1);
plot(time_temp,(joystick.input(2,start:stop))*-1,'b',time_temp,joystick.input(1,start:stop),'r',time_temp,joystick.input(4,start:stop),'g',time_temp,joystick.input(3,start:stop),'k')
title('Joystick Input','FontSize',20)
xlabel('t [s]','FontSize',20)
ylabel('Desired Joystick Input [%]','FontSize',20)
hleg=legend('$x_{in}$','$y_{in}$','$z_{in}$','$\psi_{in}$');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on

ax11(2)=subplot(2,1,2);
plot(time_temp,joystick.ref(1,start:stop),'b',time_temp,joystick.ref(2,start:stop),'r',time_temp,joystick.ref(3,start:stop),'g',time_temp,joystick.ref(6,start:stop),'k')
title('Generated Joystick Reference','FontSize',20)
xlabel('t [s]','FontSize',20)
%======Ylabel for Thrust reference======
% ylabel('Desired Thrust [N/Nm]','FontSize',20)
% hleg=legend('X [N]', 'Y[N]', 'Z[N]', 'N [Nm]');
%=====Ylabel for velocity reference======
ylabel('Desired Velocity [m/s,deg/s]','FontSize',20)
hleg=legend('u [m/s]', 'v [m/s]', 'w [m/s]', 'r [deg/s]');
set(hleg,'interpreter','latex','FontSize',20)
set(gca,'FontSize',14)
grid on

linkaxes(ax11,'x')
if tflag==0
dynamicDateTicks(ax11, 'linked')
axis auto;
end

toc
